import click
from stochasticx.auth.auth import Stochastic

stochastic = Stochastic()

@click.command(name="login")
@click.option('--username', help='Username for login')
@click.option('--password', help='Password for login')
def login(username, password):
    if username is None or password is None:
        stochastic.login()
    else:
        stochastic.login(username, password)
        
    click.echo("Login successfully")


@click.group(name="me")
def me():
    pass


@click.command(name="profile")
def profile():
    profile_info = stochastic.get_profile()
    click.echo(profile_info)
    

@click.command(name="company")
def company():
    company_info = stochastic.get_company()
    click.echo(company_info)
    

@click.command(name="usage")
def usage():
    usage_info = stochastic.get_usage_quota()
    click.echo(usage_info)
    

me.add_command(profile)
me.add_command(company)
me.add_command(usage)


import os

import stochasticx

LOGIN_URL = stochasticx.BASE_URI + "/v1/auth/login"
MODELS_URL = stochasticx.BASE_URI + "/v1/models"
OPTIMIZED_MODELS_URL = stochasticx.BASE_URI + "/v1/processedModels"
REQUEST_UPLOAD_URL = stochasticx.BASE_URI + "/v1/upload/requestUpload"
MODEL_UPLOAD_URL = stochasticx.BASE_URI + "/v1/upload/model"
DATASET_UPLOAD_URL = stochasticx.BASE_URI + "/v1/upload/dataset"
DATASETS_URL = stochasticx.BASE_URI + "/v1/datasets"
ME_URL = stochasticx.BASE_URI + "/v1/auth/me"
JOBS_URL = stochasticx.BASE_URI + "/v1/jobs"
INSTANCES_URL = stochasticx.BASE_URI + "/v1/instances"
DEPLOYMENT_URL = stochasticx.BASE_URI + "/v1/deploy"

TOKEN_AUTH_PATH = os.path.expandvars("$HOME/.stochastic/token.json")

INFERENCE_URL = stochasticx.BASE_URI + "http://infer.stochastic.ai:8000/"



import sys
import tritonclient.grpc as grpcclient

class StochasticModel:
    """Class to make inferences in a Stochastic model deployed locally"""
    
    def __init__(self, model_name: str, server_url: str = "localhost:8001"):
        """Initilizer

        :param model_name: model name
        :param server_url: URL in which the server is listening, defaults to "localhost:8001"
        """
        self.model_name = model_name
        self.server_url = server_url
        
        try:
            self.triton_client = grpcclient.InferenceServerClient(
                url=server_url,
            )

        except Exception as e:
            print("Error: Model server is not running. Aborting execution..." + str(e))
            sys.exit()
            
        model_metadata = self.triton_client.get_model_metadata(
            model_name=self.model_name, 
            model_version="1"
        )
        self.outputs = []
            
        for outp in model_metadata.outputs:
            self.outputs.append(grpcclient.InferRequestedOutput(outp.name))
            
    def predict(self, **kwargs):
        """Do the inference

        :return: the predictions
        """
        inputs = []
        outputs = []
        for name, value in kwargs.items():
            input = grpcclient.InferInput(name, value.shape, "INT32")
            input.set_data_from_numpy(value)
            inputs.append(input)
            
        # Output
        results = self.triton_client.infer(
            model_name=self.model_name,
            inputs=inputs,
            outputs=self.outputs,
            client_timeout=None,
            compression_algorithm=None) 

        for output in self.outputs:
            outputs.append(results.as_numpy(output.name()))
        
        return outputs

    def __call__(self, **kwargs):
        """It is equivalent to the predict method.

        :return: the outputs of the model
        """
        return self.predict(**kwargs)


from pathlib import Path
import os
import requests
from tqdm import tqdm
import stochasticx

from stochasticx.constants.urls import REQUEST_UPLOAD_URL, MODEL_UPLOAD_URL
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.constants.urls import DATASETS_URL, REQUEST_UPLOAD_URL, DATASET_UPLOAD_URL

class FileUtils:
    """File utils
    """
    
    @staticmethod
    def calculate_directory_size(directory_path: str):
        """Calculates the total size of a directory

        Args:
            directory_path (str): the directory path

        Returns:
            int: size in bytes
        """
        size = 0
        directory_path = Path(directory_path)
        
        assert directory_path.exists()
        
        all_files_dirs = directory_path.glob("**/*")
        for file_or_dir in all_files_dirs:
            if file_or_dir.is_file():
                size += os.path.getsize(str(file_or_dir.resolve()))
                
        return size
    

class DownloadUploadUtils:
    """Download utils
    """
    
    @staticmethod
    def _download(
        prepare_download_path: str, 
        download_path: str, 
        local_path: str
    ):
        """Download request

        Args:
            prepare_download_path (str): prepare download url
            download_path (str): download url
            local_path (str): local path where the file will be saved
        """
        auth_header = AuthUtils.get_auth_headers()
        
        # Prepare download
        r = requests.get(
            url=prepare_download_path,
            headers=auth_header
        )
        r.raise_for_status()
        download_token = r.json().get("downloadToken")
        
        # Download model
        with requests.get(
            url=download_path + "?auth_token={}".format(download_token),
            headers=auth_header,
            stream=True
        ) as r:
            r.raise_for_status()
            total_size_in_bytes = int(r.headers.get('content-length', 0))
            progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
            with open(str(local_path), 'wb') as f:
                for chunk in r.iter_content(chunk_size=8192): 
                    progress_bar.update(len(chunk))
                    f.write(chunk)
    

class ModelUtils:
    """ModelUtils
    """
    @staticmethod
    def download_model(model_id: str, local_path: str):
        """Downloads a model

        Args:
            model_id (str): model ID to be downloaded
            local_path (str): path where the model will be saved
        """
        assert model_id is not None
        prepare_download_path = stochasticx.BASE_URI + "/v1/models/{}/prepareDownload".format(model_id)
        download_path = stochasticx.BASE_URI + "/v1/models/{}/download/".format(model_id)
        
        DownloadUploadUtils._download(
            prepare_download_path,
            download_path,
            local_path
        )
        
    @staticmethod
    def download_optimized_model(model_id: str, local_path: str):
        """Downloads the optimized model

        Args:
            model_id (str): model ID
            local_path (str): path where the model will be saved
        """
        
        assert model_id is not None
        prepare_download_path = stochasticx.BASE_URI + "/v1/processedModels/{}/prepareDownload".format(model_id)
        download_path = stochasticx.BASE_URI + "/v1/processedModels/{}/download/".format(model_id)
        
        DownloadUploadUtils._download(
            prepare_download_path,
            download_path,
            local_path
        )
        
    
    @staticmethod
    def upload_model(directory_path: str, model_name: str, model_type: str):
        """Uploads a model to the platform

        Args:
            directory_path (str): directory path where the model is located
            model_name (str): model name
            model_type (str): model type

        Returns:
            str: model ID
        """
        assert directory_path is not None
        auth_header = AuthUtils.get_auth_headers()
        
        # Request upload
        directory_size = FileUtils.calculate_directory_size(directory_path)
        data = {"folderSize": directory_size}
        r = requests.post(REQUEST_UPLOAD_URL, json=data, headers=auth_header)
        r.raise_for_status()
        
        # Upload model
        base_path = Path(directory_path)
        file_list = []
        all_files_dirs = base_path.glob("**/*")
        
        for file_or_dir_path in all_files_dirs:
            if file_or_dir_path.is_file():
                file = open(str(file_or_dir_path), "rb")
                relative_file_path = str(file_or_dir_path.resolve()).replace(str(base_path.resolve()), "")[1:]
                
                file_list.append(("pyModel", (relative_file_path, file, "multipart/form-data")))
            
        request_body = {"name": model_name, "type": model_type, "folderSize": directory_size}
        r = requests.post(MODEL_UPLOAD_URL, data=request_body, files=file_list, headers=auth_header)
        r.raise_for_status()
        
        model_id = r.json().get("data").get("id")
        return model_id
    
    
class DatasetUtils:
    """Dataset utils
    """
    
    @staticmethod
    def download_dataset(dataset_id: str, local_path: str):
        """Download a dataset

        Args:
            dataset_id (str): dataset ID to be downloaded
            local_path (str): local path where the dataset will be saved
        """
        assert dataset_id is not None
        prepare_download_path = stochasticx.BASE_URI + "/v1/datasets/{}/prepareDownload".format(dataset_id)
        download_path = stochasticx.BASE_URI + "/v1/datasets/{}/download/".format(dataset_id)
        
        DownloadUploadUtils._download(
            prepare_download_path,
            download_path,
            local_path
        )
    
    @staticmethod
    def upload_dataset(directory_path: str, dataset_name: str, dataset_type: str):
        """Uploads a dataset to the Stochastic platform

        Args:
            directory_path (str): _description_
            dataset_name (str): _description_
            dataset_type (str): _description_

        Returns:
            str: dataset ID
        """
        assert directory_path is not None
        auth_header = AuthUtils.get_auth_headers()
        
        # Request upload
        directory_size = FileUtils.calculate_directory_size(directory_path)
        data = {"folderSize": directory_size}
        r = requests.post(REQUEST_UPLOAD_URL, json=data, headers=auth_header)
        r.raise_for_status()
        
        # Upload dataset
        base_path = Path(directory_path)
        all_files_dirs = base_path.glob("**/*")
        file_list = []

        for file_or_dir_path in all_files_dirs:
            if file_or_dir_path.is_file():
                file = open(str(file_or_dir_path), "rb")
                relative_file_path = str(file_or_dir_path.resolve()).replace(str(base_path.resolve()), "")[1:]
                file_list.append(("pyDataset", (relative_file_path, file, "multipart/form-data")))
                
        request_body = {"name": dataset_name, "type": dataset_type, "folderSize": directory_size}
        
        r = requests.post(DATASET_UPLOAD_URL, data=request_body, files=file_list, headers=auth_header)
        r.raise_for_status()
        
        dataset_id = r.json().get("data").get("id")
        return dataset_id


from pathlib import Path

import click
from stochasticx.datasets.datasets import Datasets, Dataset

@click.group(name="datasets")
def datasets():
    pass


@click.command(name="ls")
def ls_datasets():
    datasets = Datasets.get_datasets()
    for dataset in datasets:
        click.echo(dataset)


@click.command(name="inspect")
@click.option('--dataset_id', help='Dataset ID')
def dataset_inspect(dataset_id):
    dataset = Datasets.get_dataset(dataset_id)
    click.echo(dataset)
    click.echo(dataset.get_dataset_info())
    
    
@click.command(name="columns")
@click.option('--dataset_id', help='Dataset ID')
def dataset_columns(dataset_id):
    dataset = Datasets.get_dataset(dataset_id)
    click.echo(dataset.get_column_names())
    
    
@click.command(name="download")
@click.option('--dataset_id', help='Dataset ID')
@click.option('--local_path', help='Path where the downloaded dataset will be saved')
def dataset_download(dataset_id, local_path):
    dataset = Datasets.get_dataset(dataset_id)
    dataset.download(local_path)


@click.command(name="upload")
@click.option('--name', help='Path where the dataset to upload is located')
@click.option('--directory_path', help='Directory where the dataset to upload is located')
@click.option('--dataset_type', help='Dataset type. It should be hf, csv or json')
def dataset_upload(name, directory_path, dataset_type):
    dataset_type = dataset_type.strip()
    assert dataset_type in ["hf", "csv", "json"], "Dataset type should be hf, csv or json"
    directory_path = Path(directory_path)
    assert directory_path.exists(), "The directory path does not exist"
    
    dataset = Dataset(
        name=name,
        directory_path=directory_path,
        dataset_type=dataset_type
    )
    
    click.echo("Uploading dataset...")
    dataset.upload()
    click.echo("Dataset uploaded")


datasets.add_command(ls_datasets)
datasets.add_command(dataset_inspect)
datasets.add_command(dataset_download)
datasets.add_command(dataset_upload)
datasets.add_command(dataset_columns)


from pathlib import Path

import click
from stochasticx.models.models import Models, Model

@click.group(name="models")
def models():
    pass


@click.command(name="ls")
@click.option('--optimized', is_flag=True, help='Show only optimized models')
@click.option('--all', is_flag=True, help='Show all models')
def ls_models(optimized, all):
    if not optimized or all:
        models = Models.get_models()
        for model in models:
            click.echo(model)
    
    if optimized or all:
        models = Models.get_optimized_models()
        for model in models:
            click.echo(model)


@click.command(name="inspect")
@click.option('--model_id', help='Model ID')
def model_inspect(model_id):
    model = Models.get_model(model_id)
    click.echo(model)
    click.echo(model.get_model_info())
    
    
@click.command(name="download")
@click.option('--model_id', help='Model ID that you want to download')
@click.option('--local_path', help='Path where the downloaded model will be saved')
def model_download(model_id, local_path):
    model = Models.get_model(model_id)
    model.download(local_path)


@click.command(name="upload")
@click.option('--name', help='Path where the model to upload is located')
@click.option('--directory_path', help='Directory where the model to upload is located')
@click.option('--model_type', help='Model type. It should be hf, pt or custom')
def model_upload(name, directory_path, model_type):
    model_type = model_type.strip()
    assert model_type in ["hf", "pt", "custom"], "Model type should be hf, pt or custom"
    directory_path = Path(directory_path)
    assert directory_path.exists(), "The directory path does not exist"
    
    model = Model(
        name=name,
        directory_path=directory_path,
        model_type=model_type
    )
    
    click.echo("Uploading the model. This can take several minutes...")
    model.upload()


models.add_command(ls_models)
models.add_command(model_inspect)
models.add_command(model_download)
models.add_command(model_upload)


import click
from stochasticx.inferences.inference import (
    SequenceClassificationModel, 
    QuestionAnsweringModel, 
    SummarizationModel, 
    TranslationModel, 
    TokenClassificationModel
)


@click.group(name="inference")
def inference():
    pass


@click.command(name="sequence_classification")
@click.option('--deployment_id', help='Deployment ID')
@click.option('--text', help='Text')
def sequence_classification(deployment_id, text):
    inference_model = SequenceClassificationModel(
        deployment_id
    )
    
    labels, scores = inference_model.inference([text])
    click.echo("Labels: {}".format(labels))
    click.echo("Scores: {}".format(scores))


@click.command(name="question_answering")
@click.option('--deployment_id', help='Deployment ID')
@click.option('--question', help='Question')
@click.option('--context', help='Context')
def question_answering(deployment_id, question, context):
    inference_model = QuestionAnsweringModel(
        deployment_id
    )
    
    answer = inference_model.inference([question], [context])
    click.echo("Answer: {}".format(answer))


@click.command(name="summarization")
@click.option('--deployment_id', help='Deployment ID')
@click.option('--text', help='Text')
@click.option('--min_length', help='Min length')
@click.option('--max_length', help='Max length')
def summarization(deployment_id, text, min_length, max_length):
    inference_model = SummarizationModel(
        deployment_id
    )
    
    summary = inference_model.inference([text], [min_length], [max_length])
    click.echo("Answer: {}".format(summary))
    

@click.command(name="translation")
@click.option('--deployment_id', help='Deployment ID')
@click.option('--text', help='Text')
@click.option('--max_length', help='Max length')
def translation(deployment_id, text, max_length):
    inference_model = TranslationModel(
        deployment_id
    )
    
    translation = inference_model.inference([text], [max_length])
    click.echo("Answer: {}".format(translation))
    
    
@click.command(name="token_classification")
@click.option('--deployment_id', help='Deployment ID')
@click.option('--text', help='Text')
def token_classification(deployment_id, text):
    inference_model = TokenClassificationModel(
        deployment_id
    )
    
    tokens, tags, scores = inference_model.inference([text])
    click.echo("Tokens: {}".format(tokens))
    click.echo("Tags: {}".format(tags))
    click.echo("Scores: {}".format(scores))
    
    
inference.add_command(sequence_classification)
inference.add_command(question_answering)
inference.add_command(summarization)
inference.add_command(translation)
inference.add_command(token_classification)





class InferenceUtils:
    
    @staticmethod
    def sequence_classification(texts):
        pass
    
    @staticmethod
    def question_answering():
        pass
    
    @staticmethod
    def summarization():
        pass
    
    @staticmethod
    def translation():
        pass
    
    @staticmethod
    def token_classification():
        pass


from stochasticx.constants.urls import DATASETS_URL
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.file_utils import DatasetUtils
from stochasticx.utils.logging import configure_logger

import requests
import json
from pathlib import Path
import os


logger = configure_logger(__name__)


class DatasetType:
    """Dataset type
    """
    HUGGINGFACE = "hf"
    CSV = "csv"
    JSON = "json"


class Dataset:
    """Dataset class
    """
    
    def __init__(
        self,
        name: str,
        directory_path: str,
        dataset_type: str = DatasetType.HUGGINGFACE
    ):
        """Initializer

        Args:
            name (str): dataset name
            directory_path (str): directory in which your dataset is stored
            dataset_type (str, optional): dataset type. Defaults to DatasetType.HUGGINGFACE.
        """
        assert isinstance(name, str), "The provided name {} is not valid".format(name)
        self.name = name
        self.directory_path = directory_path
        self.dataset_type = dataset_type
        self.column_names = []
        self.dataset_id = None
        self.dataset_info = None
        self.is_uploaded = False
        
    def upload(self):
        """Uploads the dataset to the Stochastic platform
        """
        
        assert self.directory_path is not None
        
        dataset_id = DatasetUtils.upload_dataset(
            self.directory_path, 
            self.name, 
            self.dataset_type
        )
        
        self.set_id(dataset_id)
        self.is_uploaded = True
        
    def set_id(self, dataset_id: str):
        """Set the ID of the dataset

        Args:
            dataset_id (str): the new ID
        """
        self.dataset_id = dataset_id
        
    def get_id(self):
        """Returns the ID of the dataset

        Returns:
            str: the ID
        """
        return self.dataset_id
    
    def sync(self):
        """Synchronize the current dataset with the cloud
        """
        
        if self.dataset_id is not None:
            temp_ds = Datasets.get_dataset(self.dataset_id)
            self.dataset_info = temp_ds.dataset_info
            self.name = temp_ds.name
    
    def get_dataset_info(self):
        """Gets the dataset information

        Returns:
            dict: dataset information
        """
        
        self.sync()
        
        return self.dataset_info
                
    def set_dataset_info(self, dataset_info):
        """Sets the dataset information

        Args:
            dataset_info (dict): dataset information
        """
        self.dataset_info = dataset_info
        
    def get_column_names(self):
        """Get column names of the dataset

        Returns:
            List[str]: the column names
        """
        if self.dataset_id is not None:
            temp_ds = Datasets.get_dataset(self.dataset_id)
            self.column_names = temp_ds.column_names
            
        return self.column_names
        
    def set_column_names(self, column_names):
        """Set the column names of the dataset

        Args:
            column_names (List[str]): new column names
        """
        self.column_names = column_names
    
    def download(self, local_path: str):
        """Downloads the dataset

        Args:
            local_path (str): local path where the dataset is saved
        """
        assert self.dataset_id is not None
        DatasetUtils.download_dataset(self.dataset_id, local_path)
                    
    def __str__(self):
        """Method to convert the object to string

        Returns:
            str: string
        """
        return "ID: {} ; Dataset name: {} ; Directory path: {} ; Dataset type: {} ; Uploaded: {}".format(
            self.dataset_id,
            self.name,
            self.directory_path,
            self.dataset_type,
            self.is_uploaded
        )


class Datasets:
    """Class to get the uploaded datasets
    """
    
    @staticmethod
    def get_dataset(dataset_id: str):
        """Get dataset by ID

        Args:
            dataset_id (str): ID

        Returns:
            Dataset: the dataset
        """
        url = DATASETS_URL + "/{}".format(dataset_id)

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(url, headers=auth_header)
        dataset_data = response.json().get("data")

        if dataset_data is not None:
            ds = Dataset(
                name=dataset_data.get("name"),
                directory_path=None,
                dataset_type=dataset_data.get("type")
            )
            
            ds.set_id(dataset_data.get("id"))
            ds.set_dataset_info(dataset_data.get("datasetInfo"))
            if dataset_data.get("datasetInfo") is not None:
                ds.set_column_names(dataset_data.get("datasetInfo").get("column_names"))
            ds.is_uploaded = True

            return ds

        return None
    
    @staticmethod
    def get_datasets():
        """Gets all the datasets

        Returns:
            List[Dataset]: all the uploaded datasets
        """
        datasets = []

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(DATASETS_URL, headers=auth_header)
        datasets_data = response.json().get("data")

        if datasets_data is not None:
            for dataset_data in datasets_data:
                ds = Dataset(
                    name=dataset_data.get("name"),
                    directory_path=None,
                    dataset_type=dataset_data.get("type")
                )
                
                ds.set_id(dataset_data.get("id"))
                ds.set_dataset_info(dataset_data.get("datasetInfo"))
                if dataset_data.get("datasetInfo") is not None:
                    ds.set_column_names(dataset_data.get("datasetInfo").get("column_names"))
                ds.is_uploaded = True
                
                datasets.append(ds)

        return datasets


import requests
from datetime import datetime

from stochasticx.constants.urls import INFERENCE_URL, INSTANCES_URL, DEPLOYMENT_URL
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.models.models import OptimizedModel, Models
from stochasticx.utils.logging import configure_logger

logger = configure_logger(__name__)


class InferenceTaskType:
    def __init__(self):
        pass
    
    def return_parameters(self):
        raise NotImplementedError()
    

class SequenceClassificationInfTask(InferenceTaskType):
    def __init__(
        self,
        max_batch_size=8,
        max_seq_length=128
    ):
        self.max_batch_size = max_batch_size
        self.max_seq_length = max_seq_length
    
    def return_parameters(self):
        return {
            "max_batch_size": self.max_batch_size,
            "max_seq_length": self.max_seq_length
        }   
    

class QuestionAnsweringInfTask(InferenceTaskType):
    def __init__(
        self,
        max_batch_size=8,
        max_seq_length=128
    ):
        self.max_batch_size = max_batch_size
        self.max_seq_length = max_seq_length
    
    def return_parameters(self):
        return {
            "max_batch_size": self.max_batch_size,
            "max_seq_length": self.max_seq_length
        }    
    
    
class SummarizationInfTask(InferenceTaskType):
    def __init__(
        self,
        max_batch_size=8,
        max_source_length=128,
        source_prefix='',
        lang='en'
    ):
        self.max_batch_size = max_batch_size
        self.max_source_length = max_source_length
        self.source_prefix = source_prefix
        self.lang = lang
    
    def return_parameters(self):
        return {
            "max_batch_size": self.max_batch_size,
            "max_source_length": self.max_source_length,
            "source_prefix": self.source_prefix,
            "lang": self.lang
        }
    
    
class TranslationInfTask(InferenceTaskType):
    def __init__(
        self,
        max_batch_size=8,
        max_source_length=128,
        source_prefix='',
        src_lang='en',
        tgt_lang='de'
    ):
        self.max_batch_size = max_batch_size
        self.max_source_length = max_source_length
        self.source_prefix = source_prefix
        self.src_lang = src_lang
        self.tgt_lang = tgt_lang
    
    def return_parameters(self):
        return {
            "max_batch_size": self.max_batch_size,
            "max_source_length": self.max_source_length,
            "source_prefix": self.source_prefix,
            "src_lang": self.src_lang,
            "tgt_lang": self.tgt_lang
        }   
    
    
class TokenClassificationInfTask(InferenceTaskType):
    def __init__(
        self,
        max_batch_size=8,
        max_seq_length=128
    ):
        self.max_batch_size = max_batch_size
        self.max_seq_length = max_seq_length
    
    def return_parameters(self):
        return {
            "max_batch_size": self.max_batch_size,
            "max_seq_length": self.max_seq_length
        }   


class InstanceTypes:
    g4dn_xlarge = "g4dn.xlarge"
    c5_2xlarge = "c5.2xlarge"
    c5_12xlarge = "c5.12xlarge"


class Instance:
    def __init__(
        self,
        id,
        name,
        cost_per_hour,
        cost_per_month,
        spot_cost,
        storage,
        vcpus,
        memory,
        network
    ):
        self.id = id
        self.name = name
        self.cost_per_hour = cost_per_hour
        self.cost_per_month = cost_per_month
        self.spot_cost = spot_cost
        self.storage = storage
        self.vcpus = vcpus
        self.memory = memory
        self.network = network
        
    def __str__(self):
        return "ID: {} ; Name: {} ; Cost/hour: {} ; Cost/month: {} ; Spot cost: {} ; Storage: {} ; vCPUs: {} ; Memory: {} ; Network: {}".format(
            self.id,
            self.name,
            self.cost_per_hour,
            self.cost_per_month,
            self.spot_cost,
            self.storage,
            self.vcpus,
            self.memory,
            self.network
        )
        

class Deployment:
    def __init__(
        self
    ):        
        pass
            
    def populate_data(
        self,
        id,
        task_type,
        created_at,
        instance,
        optimized_model,
        status,
        parameters,
        start,
        end,
        endpoint=None,
        api_key=None
    ):
        self.id = id
        self.task_type = task_type
        self.created_at = created_at
        self.instance = instance
        self.optimized_model = optimized_model
        self.status = status
        self.parameters = parameters
        
        self.start = None
        if start is not None:
            self.start = datetime.utcfromtimestamp(
                int(start)
            ).strftime('%Y-%m-%d %H:%M:%S')
            
        self.end = None
        if end is not None:
            self.end = datetime.utcfromtimestamp(
                int(end)
            ).strftime('%Y-%m-%d %H:%M:%S')
            
        self.endpoint = endpoint
        self.api_key = api_key
        
    def get_endpoint(self):
        self.sync()
        return self.endpoint
    
    def get_api_key(self):
        self.sync()
        return self.api_key
        
    def sync(self):
        synced_deployment = Deployments.get_deployment(self.id)
        self.task_type = synced_deployment.task_type
        self.created_at = synced_deployment.created_at
        self.instance = synced_deployment.instance
        self.optimized_model = synced_deployment.optimized_model
        self.parameters = synced_deployment.parameters
        self.endpoint = synced_deployment.endpoint
        self.api_key = synced_deployment.api_key
        self.start = synced_deployment.start
        self.end = synced_deployment.end
        self.status = synced_deployment.status
    
    def get_status(self):
        self.sync()
        return self.status 
    
    def get_instance(self):
        self.sync()
        return self.instance
    
    def get_optimized_model(self):
        self.sync()
        return self.optimized_model
    
    def get_task_type(self):
        self.sync()
        return self.task_type
    
    def get_id(self):
        return self.id
    
    def start_inference(
        self,
        model,
        task_type,
        instance_type=InstanceTypes.g4dn_xlarge
    ):        
        auth_header = AuthUtils.get_auth_headers()
        r = requests.post(DEPLOYMENT_URL, headers=auth_header, json={
            "modelId": model.id,
            "instanceType": instance_type,
            "userParams": task_type.return_parameters()
        })
        r.raise_for_status()
        self.id = r.json().get("data").get("id")
    
    def delete(self):
        if self.get_status() == "running":
            url = DEPLOYMENT_URL + "/{}".format(self.id)
            auth_header = AuthUtils.get_auth_headers()
            r = requests.delete(url, headers=auth_header)
            r.raise_for_status()
        else:
            logger.warning("You cannot delete a deployment until it is running")
    
    def __str__(self):
        return "ID: {} ; Created at: {} ; Status: {}; Parameters: {} ; Start: {} ; End: {}".format(
            self.id,
            self.created_at,
            self.status,
            self.parameters,
            self.start,
            self.end
        )
        

class Instances:
    
    @staticmethod
    def get_instance_types():
        instances = []
        
        auth_header = AuthUtils.get_auth_headers()
        r = requests.get(INSTANCES_URL, headers=auth_header)
        r.raise_for_status()
        
        data = r.json()
        
        if data.get("ec2Instances") is not None:
            for instance_data in data["ec2Instances"]:
                instance = Instance(
                    id=instance_data["id"],
                    name=instance_data["instanceType"],
                    cost_per_hour=instance_data["hourlyCost"],
                    memory=instance_data["memory"],
                    cost_per_month=instance_data["monthlyCost"],
                    network=instance_data["network"],
                    spot_cost=instance_data["spotCost"],
                    storage=instance_data["storage"],
                    vcpus=instance_data["vcpus"],
                )
                
                instances.append(instance)
        
        return instances
                
    @staticmethod
    def get_instance_type_by_name(name):
        instances = Instances.get_instance_types()
        
        for instance in instances:
            if instance.name == name:
                return instance
                

class DeploymentUtils:
    
    @staticmethod
    def create_deployment(deployment_data):
        optimized_model_id = deployment_data["model"]["id"]
        optimized_model = Models.get_optimized_model(optimized_model_id)
        
        instance_data = deployment_data.get("instance")
        
        instance = None
        if instance_data is not None:
            instance = Instance(
                id=instance_data["id"],
                name=instance_data["instanceType"],
                cost_per_hour=instance_data["hourlyCost"],
                memory=instance_data["memory"],
                cost_per_month=instance_data["monthlyCost"],
                network=instance_data["network"],
                spot_cost=instance_data["spotCost"],
                storage=instance_data["storage"],
                vcpus=instance_data["vcpus"],
            )
        
        start = None
        end = None
        if deployment_data.get("utilization") is not None:
            start = deployment_data["utilization"].get("start")
            end = deployment_data["utilization"].get("end")
            
            
        endpoint = None
        api_key = None
        resources = deployment_data.get("resources")
        if resources is not None:
            endpoint = INFERENCE_URL + resources.get("route")[2:]
            api_key = resources.get("apiKey")
        
        deployment = Deployment()
        deployment.populate_data(
            id=deployment_data["id"],
            task_type=deployment_data["job"]["taskType"],
            created_at=deployment_data["createdAt"],
            instance=instance,
            optimized_model=optimized_model,
            status=deployment_data["status"],
            parameters=deployment_data["userParams"],
            start=start,
            end=end,
            endpoint=endpoint,
            api_key=api_key
        )
        
        return deployment


class Deployments:        
    
    @staticmethod
    def get_deployments():
        deployments = []
        
        auth_header = AuthUtils.get_auth_headers()
        r = requests.get(DEPLOYMENT_URL, headers=auth_header)
        r.raise_for_status()
        
        data = r.json()
        
        if data["data"] is not None:
            for deployment_data in data["data"]:
                deployment = DeploymentUtils.create_deployment(deployment_data)
                deployments.append(deployment)
                
        return deployments
    
    @staticmethod
    def get_deployment(id):
        url = DEPLOYMENT_URL + "/{}".format(id)
        
        auth_header = AuthUtils.get_auth_headers()
        r = requests.get(url, headers=auth_header)
        r.raise_for_status()
        
        data = r.json()
        deployment_data = data["deployedModel"]
        if deployment_data is not None:
            deployment = DeploymentUtils.create_deployment(deployment_data)
            return deployment
            
        return None
            

        


import os
import json
import requests
from pathlib import Path
from stochasticx.constants.urls import LOGIN_URL, TOKEN_AUTH_PATH


class LoginUtils:
    @staticmethod
    def login_request(username, password):
        """Private method to do the login request

        Args:
            username (str): username
            password (str): password

        Returns:
            str: token
        """
        login_info = {"email": username, "password": password}

        response = requests.post(LOGIN_URL, data=login_info)
        response.raise_for_status()

        response_data = response.json()
        token = response_data["tokens"]["access"]["token"]
        
        # Save token in ENV var
        os.environ["STOCHASTIC_TOKEN"] = token
        
        # Save token in a file
        token_path = Path(TOKEN_AUTH_PATH).resolve()
        
        if not token_path.exists():
            token_path.parent.mkdir(parents=True, exist_ok=True)
            
        with open(str(token_path), 'w', encoding='utf-8') as f:
            json.dump({
                "token": token
            }, f, ensure_ascii=False, indent=4)
        
        return token

class AuthUtils:
    
    @staticmethod
    def get_auth_headers():
        """Get authentication headers from the webapp

        Raises:
            ValueError: if token not found in the following ENV variable STOCHASTIC_TOKEN

        Returns:
            dict: auth headers
        """
        token = os.getenv("STOCHASTIC_TOKEN")
        
        if token is None:
            f = open(TOKEN_AUTH_PATH)
            data = json.load(f)
            token = data['token']

        
        if token is None:
            raise ValueError("Do login before running any request")
        else:
            return {
                'Authorization': 'Bearer ' + token
            }


import os
import requests
from stochasticx.constants.urls import ME_URL
from stochasticx.utils.auth_utils import AuthUtils, LoginUtils
import ipywidgets as widgets
from IPython.display import display


class UsageQuota:
    """Class to get your current usage quota and your limits
    """
    
    def __init__(
        self,
        concurrentJobLimit: int,
        monthlyJobLimit: int,
        storageUsed: int,
        storageLimit: int
    ):
        """Initializer

        Args:
            concurrentJobLimit (int): concurrent job limit
            monthlyJobLimit (int): monthly job limit
            storageUsed (int): storage used
            storageLimit (int): storage limit
        """
        
        self.concurrentJobLimit = concurrentJobLimit
        self.monthlyJobLimit = monthlyJobLimit
        self.storageUsed = storageUsed
        self.storageLimit = storageLimit
        
    def __str__(self):
        """Convert the object to a string

        Returns:
            str
        """
        return "Concurrent job limit {} ; Monthly job limit: {} ; Storage used: {} ; Storage limit: {}".format(
            self.concurrentJobLimit,
            self.monthlyJobLimit,
            self.storageUsed,
            self.storageLimit
        )
        

class Company:
    """Class to store information about the company of the user
    """
    
    def __init__(
        self,
        id: str,
        name: str,
        website: str
    ):
        """Initializer

        Args:
            id (str): ID
            name (str): company name
            website (str): website
        """
        
        self.id = id
        self.name = name
        self.website = website
        
    def __str__(self):
        """Method to convert the object to a string

        Returns:
            str
        """
        return "ID: {} ; Company name: {} ; Website: {}".format(
            self.id,
            self.name,
            self.website
        )
        

class User:
    """Class to store information about the user
    """
    
    def __init__(
        self, 
        id: str, 
        first_name: str, 
        last_name: str, 
        email: str, 
        role: str
    ):
        """Initializer

        Args:
            id (str): ID
            first_name (str): first name
            last_name (str): last name
            email (str): email
            role (str): role
        """
        
        self.id = id
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.role = role
        
    def __str__(self):
        """Method to conver the object to a string

        Returns:
            str
        """
        return "ID: {} ; First name: {} ; Last name: {} ; Email: {} ; Role: {}".format(
            self.id,
            self.first_name,
            self.last_name,
            self.email,
            self.role
        )



class Stochastic:
    """Class to do login and store the information of the current user
    """
    
    def __init__(
        self
    ):
        """Initializer
        """
        
        self.username = os.getenv("STOCHASTIC_USER")
        self.password = os.getenv("STOCHASTIC_PASSWORD")
        self.company_info = None
        self.usage_quota = None
        self.user_info = None

    def login(self):
        """Do login
        """
        
        LoginUtils.login_request(self.username, self.password)
        
    def login(self, username, password):
        LoginUtils.login_request(username, password)
    
    def notebook_login(self):
        """Allows you to login in a Jupyter Notebook
        """
        
        username = widgets.Text(description="Username")
        password = widgets.Password(description="Password")
        output = widgets.Output()
        login_button = widgets.Button(description="Login")
        
        display(username, password, login_button, output)

        def on_button_clicked(b):
            with output:
                logged_in = LoginUtils.login_request(username.value, password.value)
                if logged_in:
                    print("Login successfully")
                else:
                    print("Username or password is not correct")

        login_button.on_click(on_button_clicked)
        
    def _initilize_profile(self):
        """Populate the profile with data
        """
        
        auth_header = AuthUtils.get_auth_headers()
        
        response = requests.get(ME_URL, headers=auth_header)
        data = response.json()
        
        if data.get("company") is not None:
            self.company_info = Company(
                id=data.get("company").get("id"),
                name=data.get("company").get("name"),
                website=data.get("company").get("website")
            )
            
            self.usage_quota = UsageQuota(
                concurrentJobLimit=data.get("company").get("concurrentJobLimit"),
                monthlyJobLimit=data.get("company").get("monthlyJobLimit"),
                storageUsed=data.get("company").get("storageUsed"),
                storageLimit=data.get("company").get("storageLimit")
            )
        
        self.user_info = User(
            id=data.get("id"),
            first_name=data.get("firstName"),
            last_name=data.get("lastName"),
            email=data.get("email"),
            role=data.get("role")
        )
        
    def get_profile(self):
        """Returns profile information

        Returns:
            User: user information
        """
        return self.user_info
    
    def get_company(self):
        """Returns the company information

        Returns:
            Company: company information
        """
        return self.company_info
    
    def get_usage_quota(self):
        """Return the usage quota

        Returns:
            UsageQuota: usage quota
        """
        return self.usage_quota


from stochasticx.constants.urls import MODELS_URL, OPTIMIZED_MODELS_URL
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.file_utils import ModelUtils
import requests


class ModelType:
    """Model type
    """
    
    HUGGINGFACE = "hf"
    PYTORCH = "pt"
    ONNX = "onnx"
    CUSTOM = "custom"


class Model:
    """Model class
    """
    
    def __init__(
        self,
        name: str,
        directory_path: str,
        model_type: str = ModelType.HUGGINGFACE
    ):
        """Initializer

        Args:
            name (str): model name
            directory_path (str): directory path where the model is located
            model_type (str, optional): model type. Defaults to ModelType.HUGGINGFACE.
        """
        assert isinstance(name, str), "The provided name {} is not valid".format(name)
        
        self.name = name
        self.directory_path = directory_path
        self.model_type = model_type
        self.model_id = None
        self.model_info = None
        self.is_uploaded = False
        
    def upload(self):
        """Upload the model to the stochastic platform
        """
        
        model_id = ModelUtils.upload_model(
            self.directory_path, 
            self.name, 
            self.model_type
        )
        
        self.set_id(model_id)
        self.is_uploaded = True
        
    def set_id(self, model_id: str):
        """Sets the model ID

        Args:
            model_id (str): new ID
        """
        self.model_id = model_id
        
    def sync(self):
        """Syncs the model information with the stochastic platform
        """
        
        if self.model_id is not None:
            temp_model = Models.get_model(self.model_id)
            self.name = temp_model.name
            self.model_info = temp_model.model_info
            self.is_uploaded = True
    
    def get_id(self):
        """Gets the model ID

        Returns:
            str: the model ID
        """
        return self.model_id
    
    def get_model_info(self):
        """Gets the model information

        Returns:
            dict: model information
        """
        
        self.sync()
        
        return self.model_info
                
    def set_model_info(self, model_info):
        """Sets the model information

        Args:
            model_info (dict): model info
        """
        self.model_info = model_info
    
    def download(self, local_path: str):
        """Download the model for the Stochastic platform

        Args:
            local_path (str): path where the model will be saved
        """
        assert self.model_id is not None
        ModelUtils.download_model(self.model_id, local_path)
                    
    def __str__(self):
        """Convert the object to string

        Returns:
            str
        """
        return "Model ID: {} ; Name: {} ; Directory path: {} ; Model type: {} ; Uploaded: {}".format(
            self.model_id,
            self.name,
            self.directory_path,
            self.model_type,
            self.is_uploaded
        )
    

class OptimizedModel:
    """Optimized model class
    """
    
    def __init__(
        self,
        id: str,
        name: str,
        type: str,
        size: str,
        benchmark_results: dict
    ):
        """Initializer

        Args:
            id (str): model ID
            name (str): model name
            type (str): model type
            size (str): model size in bytes
            benchmark_results (dict): benchmark results
        """
        
        self.id = id
        self.name = name
        self.type = type
        self.size = size
        self.benchmark_results = benchmark_results
        
    def download(self, local_path: str):
        """Downloads the model from the Stochastic platform

        Args:
            local_path (str): path where the model will be saved
        """
        
        ModelUtils.download_optimized_model(self.id, local_path)
    
    def get_benchmark_results(self):
        """Get benchmark results

        Returns:
            dict: benchmark results
        """
        return self.benchmark_results
    
    def __str__(self):
        """Converts the object to a string

        Returns:
            str
        """
        
        return "Optimized model ID: {} ; Name: {} ; Type: {} ; Size: {} ".format(
            self.id,
            self.name,
            self.type,
            self.size
        )
        

class Models:
    """Class to get the models uploaded in the Stochastic platform
    """
    
    @staticmethod
    def get_model(model_id: str):
        """Gets the model from the Stochastic platform

        Args:
            model_id (str): model ID

        Returns:
            Model: model
        """
        
        url = MODELS_URL + "/{}".format(model_id)

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(url, headers=auth_header)
        model_data = response.json().get("data")
        
        if model_data is not None:
            model = Model(
                name=model_data.get("name"),
                directory_path=None,
                model_type=model_data.get("type")
            )
            
            model.set_id(model_data.get("id"))
            model.set_model_info(model_data.get("modelInfo"))
            model.is_uploaded = True
            
            return model
        
        return None
    
    @staticmethod
    def get_models():
        """Gets all the models from the stochastic platform

        Returns:
            List[Model]: list of models
        """
        models = []

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(MODELS_URL, headers=auth_header)
        models_data = response.json().get("data")

        if models_data is not None:
            for model_data in models_data:
                model = Model(
                    name=model_data.get("name"),
                    directory_path=None,
                    model_type=model_data.get("type")
                )
                
                model.set_id(model_data.get("id"))
                model.set_model_info(model_data.get("modelInfo"))
                model.is_uploaded = True
                models.append(model)

        return models
    
    @staticmethod
    def get_optimized_models():
        """Gets all the optimized models from the Stochastic platform

        Returns:
            List[OptimizedModel]: list of optimized models
        """
        
        models = []

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(OPTIMIZED_MODELS_URL, headers=auth_header)
        models_data = response.json().get("optimizedModels")

        if models_data is not None:
            for model_data in models_data:
                model = OptimizedModel(
                    id=model_data.get("id"),
                    name=model_data.get("userGivenName"),
                    type=model_data.get("type"),
                    size=model_data.get("size"),
                    benchmark_results=model_data.get("result").get("results")
                )

                models.append(model)

        return models
    
    @staticmethod
    def get_optimized_model(model_id: str):
        """Gets an optimized model from the Stochastic plarform

        Args:
            model_id (str): model ID

        Returns:
            OptimizedModel: optimized model
        """
        
        url = OPTIMIZED_MODELS_URL + "/{}".format(model_id)

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(url, headers=auth_header)
        model_data = response.json().get("data")
                
        if model_data is not None and len(model_data) > 0:
            model = OptimizedModel(
                id=model_data.get("id"),
                name=model_data.get("userGivenName"),
                type=model_data.get("type"),
                size=model_data.get("size"),
                benchmark_results=model_data.get("result").get("results")
            )
            
            return model
        
        return None


from stochasticx.deployment.deployments import Deployments
from stochasticx.utils.logging import configure_logger

import requests

logger = configure_logger(__name__)


class InferenceStatus:
    ALIVE = "alive"
    READY = "ready"
    DOWN = "down"


class InferenceModel:
    def __init__(
        self,
        deployment_id
    ):
        self.deployment_id = deployment_id
        self.deployment = Deployments.get_deployment(self.deployment_id)
        self.task_type = self.deployment.get_task_type()
        deployment_status = self.deployment.get_status()
        
        assert self.deployment is not None, "This deployment is not available"
        
        assert deployment_status == "running", "InferenceModel object cannot be created until the deployment has been successfully created"
        
    def get_status(self):
        return self.deployment.get_status()
        
    def get_task_type(self):
        return self.task_type
    
    def inference(self, *args, **kwargs):
        raise NotImplementedError("This method should be implemented by the child class")
    
    
class SequenceClassificationModel(InferenceModel):
    def __init__(
        self,
        deployment_id
    ):
        super().__init__(deployment_id)
        
    def format_inputs(self, texts):
        return {
            "inputs": [{
                "name": "text",
                "datatype": "BYTES",
                "shape": [1, len(texts)],
                "data": texts
            }]
        }
    
    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            labels, scores = outputs[0], outputs[1]
            return labels.get("data"), scores.get("data")
        
    
    def inference(self, texts):
        model_inputs = self.format_inputs(texts)
        endpoint_url = self.deployment.get_endpoint()
        api_key = self.deployment.get_api_key()
        
        assert endpoint_url is not None
        assert api_key is not None
        
        auth_header = {
            "apiKey": api_key
        }
        
        r = requests.post(endpoint_url, headers=auth_header, json=model_inputs)
        r.raise_for_status()
        
        outputs = r.json()
        labels, scores = self.format_output(outputs)
        
        return labels, scores
    

class QuestionAnsweringModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)
        
    def format_inputs(self, questions, contexts):
        return {
            "inputs": [
                {
                    "name": "question",
                    "datatype": "BYTES",
                    "shape": [1, len(questions)],
                    "data": questions
                },
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(contexts)],
                    "data": contexts
                }
            ]
        }
    
    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            answers = outputs[0]
            return answers.get("data")
        
    def inference(self, questions, contexts):
        model_inputs = self.format_inputs(questions, contexts)
        endpoint_url = self.deployment.get_endpoint()
        api_key = self.deployment.get_api_key()
        
        assert endpoint_url is not None
        assert api_key is not None
        
        auth_header = {
            "apiKey": api_key
        }
        
        r = requests.post(endpoint_url, headers=auth_header, json=model_inputs)
        r.raise_for_status()
        
        outputs = r.json()
        answers = self.format_output(outputs)
        
        return answers
        
        
class SummarizationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)
        
    def format_inputs(self, texts, min_lengths, max_lengths):
        return {
            "inputs":[
                {
                    "name":"text",
                    "datatype":"BYTES",
                    "shape":[1, len(texts)],
                    "data":texts
                },
                {
                    "name":"min_length",
                    "datatype":"INT32",
                    "shape":[1, len(min_lengths)],
                    "data": min_lengths
                },
                {
                    "name":"max_length",
                    "datatype":"INT32",
                    "shape":[1, len(max_lengths)],
                    "data": max_lengths
                }
            ]
        }
    
    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            summaries = outputs[0]
            return summaries.get("data")
        
    def inference(self, texts, min_lengths, max_lengths):
        model_inputs = self.format_inputs(texts, min_lengths, max_lengths)
        endpoint_url = self.deployment.get_endpoint()
        api_key = self.deployment.get_api_key()
        
        assert endpoint_url is not None
        assert api_key is not None
        
        auth_header = {
            "apiKey": api_key
        }
        
        r = requests.post(endpoint_url, headers=auth_header, json=model_inputs)
        r.raise_for_status()
        
        outputs = r.json()
        summaries = self.format_output(outputs)
        
        return summaries
       

class TranslationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)
        
    def format_inputs(self, texts, max_lengths):
        return {
            "inputs":[
                {
                    "name":"text",
                    "datatype":"BYTES",
                    "shape":[1, len(texts)],
                    "data":texts
                },
                {
                    "name":"max_length",
                    "datatype":"INT32",
                    "shape":[1, len(max_lengths)],
                    "data": max_lengths
                }
            ]
        }
    
    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            translations = outputs[0]
            return translations.get("data")
        
    def inference(self, texts, max_lengths):
        model_inputs = self.format_inputs(texts, max_lengths)
        endpoint_url = self.deployment.get_endpoint()
        api_key = self.deployment.get_api_key()
        
        assert endpoint_url is not None
        assert api_key is not None
        
        auth_header = {
            "apiKey": api_key
        }
        
        r = requests.post(endpoint_url, headers=auth_header, json=model_inputs)
        r.raise_for_status()
        
        outputs = r.json()
        translations = self.format_output(outputs)
        
        return translations
    

class TokenClassificationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)
        
    def format_inputs(self, texts):
        return {
            "inputs": [
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(texts)],
                    "data": texts
                }
            ]
        }
    
    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            tokens, tags, scores = outputs[0], outputs[1], outputs[2]
            return tokens.get("data"), tags.get("data"), scores.get("data")
        
    def inference(self, texts):
        model_inputs = self.format_inputs(texts)
        endpoint_url = self.deployment.get_endpoint()
        api_key = self.deployment.get_api_key()
        
        assert endpoint_url is not None
        assert api_key is not None
        
        auth_header = {
            "apiKey": api_key
        }
        
        r = requests.post(endpoint_url, headers=auth_header, json=model_inputs)
        r.raise_for_status()
        
        outputs = r.json()
        tokens, tags, scores = self.format_output(outputs)
        
        return tokens, tags, scores


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
   long_description = fh.read()

install_requires = [
   "requests",
   "jwt",
   "Click",
   "tqdm",
   "ipywidgets",
   "tritonclient[all]"
]

"""scripts=[
   "bin/stochasticx"
]"""

setuptools.setup(
   name='stochasticx',
   version='0.0.2',
   author='Marcos Rivera Martínez, Sarthak Langde, Glenn Ko, Subhash G N, Jolina Li',
   author_email='marcos.rm@stochastic.ai, sarthak.langde@stochastic.ai, glenn@stochastic.ai, subhash.gn@stochastic.ai, jolina.li@mail.utoronto.ca',
   description='Stochastic client library',
   long_description=long_description,
   long_description_content_type="text/markdown",
   url="https://github.com/stochasticai/stochasticx",
   classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent"
   ],
   package_dir={"": "src"},
   packages=setuptools.find_packages(where="src"),
   python_requires=">=3.7",
   install_requires=install_requires,
   entry_points='''
      [console_scripts]
      stochasticx=stochasticx.scripts.stochasticx:cli
   ''', 
)


from pathlib import Path

import click

from stochasticx.deployment.deployments import (
    Deployment,
    Deployments,
    Instances,
    SequenceClassificationInfTask,
    QuestionAnsweringInfTask,
    TranslationInfTask,
    SummarizationInfTask,
    TokenClassificationInfTask
)

from stochasticx.models.models import Models

@click.group(name="deployments")
def deployments():
    pass


@click.command(name="ls")
def ls_deployments():
    deployments = Deployments.get_deployments()
    for deployment in deployments:
        click.echo(deployment)
        
        
@click.command(name="inspect")
@click.option('--deployment_id', help='Deployment ID')
def inspect_deployment(deployment_id):
    deployment = Deployments.get_deployment(deployment_id)
    click.echo(deployment)
    
    if deployment is not None:
        click.echo("Instance used: {}".format(deployment.get_instance()))
        click.echo("Deployed model: {}".format(deployment.get_optimized_model()))
        
        
@click.command(name="status")
@click.option('--deployment_id', help='Deployment ID')
def deployment_status(deployment_id):
    deployment = Deployments.get_deployment(deployment_id)
    click.echo(deployment.get_status())
    
    
@click.group(name="deploy")
def deploy():
    pass

    
@click.command(name="sequence_classification")
@click.option('--model_id', help='Model ID')
@click.option('--instance_type', help='Instance type')
@click.option('--max_batch_size', default=8, show_default=True, help='Maximum batch size')
@click.option('--max_seq_length', default=128, show_default=True, help='Maximum source length')
def sequence_classification(
    model_id,
    instance_type,
    max_batch_size,
    max_seq_length
):
    task_type = SequenceClassificationInfTask(
        max_batch_size=max_batch_size,
        max_seq_length=max_seq_length
    )
    
    model = Models.get_optimized_model(model_id)
    
    deployment = Deployment()
    deployment.start_inference(
        model=model,
        task_type=task_type,
        instance_type=instance_type
    )


@click.command(name="question_answering")
@click.option('--model_id', help='Model ID')
@click.option('--instance_type', help='Instance type')
@click.option('--max_batch_size', default=8, show_default=True, help='Maximum batch size')
@click.option('--max_seq_length', default=128, show_default=True, help='Maximum sequence length')
def question_answering(
    model_id,
    instance_type,
    max_batch_size,
    max_seq_length
):
    task_type = QuestionAnsweringInfTask(
        max_batch_size=max_batch_size,
        max_seq_length=max_seq_length
    )
    
    model = Models.get_optimized_model(model_id)
    
    deployment = Deployment()
    deployment.start_inference(
        model=model,
        task_type=task_type,
        instance_type=instance_type
    )


@click.command(name="summarization")
@click.option('--model_id', help='Model ID')
@click.option('--instance_type', help='Instance type')
@click.option('--max_batch_size', default=8, show_default=True, help='Maximum batch size')
@click.option('--max_source_length', default=128, show_default=True, help='Maximum source length')
@click.option('--source_prefix', default='', show_default=True, help='Source prefix')
@click.option('--lang', default='en', show_default=True, help='Language')
def summarization(
    model_id,
    instance_type,
    max_batch_size,
    max_source_length,
    source_prefix,
    lang
):
    task_type = SummarizationInfTask(
        max_batch_size=max_batch_size,
        max_source_length=max_source_length,
        source_prefix=source_prefix,
        lang=lang
    )
    
    model = Models.get_optimized_model(model_id)
    
    deployment = Deployment()
    deployment.start_inference(
        model=model,
        task_type=task_type,
        instance_type=instance_type
    )


@click.command(name="translation")
@click.option('--model_id', help='Model ID')
@click.option('--instance_type', help='Instance type')
@click.option('--max_batch_size', default=8, show_default=True, help='Maximum batch size')
@click.option('--max_source_length', default=128, show_default=True, help='Maximum source length')
@click.option('--source_prefix', default='', show_default=True, help='Source prefix')
@click.option('--src_lang', default='en', show_default=True, help='Source language')
@click.option('--tgt_lang', default='de', show_default=True, help='Target language')
def translation(
    model_id,
    instance_type,
    max_batch_size,
    max_source_length,
    source_prefix,
    src_lang,
    tgt_lang
):
    task_type = TranslationInfTask(
        max_batch_size=max_batch_size,
        max_source_length=max_source_length,
        source_prefix=source_prefix,
        src_lang=src_lang,
        tgt_lang=tgt_lang
    )
    
    model = Models.get_optimized_model(model_id)
    
    deployment = Deployment()
    deployment.start_inference(
        model=model,
        task_type=task_type,
        instance_type=instance_type
    )


@click.command(name="token_classification")
@click.option('--model_id', help='Model ID')
@click.option('--instance_type', help='Model ID')
@click.option('--max_batch_size', default=8, show_default=True, help='Maximum batch size')
@click.option('--max_seq_length', default=128, show_default=True, help='Maximum source length')
def token_classification(
    model_id,
    instance_type,
    max_batch_size,
    max_seq_length
):
    task_type = TokenClassificationInfTask(
        max_batch_size=max_batch_size,
        max_seq_length=max_seq_length
    )
    
    model = Models.get_optimized_model(model_id)
    
    deployment = Deployment()
    deployment.start_inference(
        model=model,
        task_type=task_type,
        instance_type=instance_type
    )


        
@click.group(name="instances")
def instances():
    pass


@click.command(name="ls")
def ls_instances():
    instances = Instances.get_instance_types()
    for instance in instances:
        click.echo(instance)
        
        
deploy.add_command(sequence_classification)
deploy.add_command(question_answering)
deploy.add_command(token_classification)
deploy.add_command(summarization)
deploy.add_command(translation)

deployments.add_command(ls_deployments)
deployments.add_command(inspect_deployment)
deployments.add_command(deployment_status)
deployments.add_command(deploy)

instances.add_command(ls_instances)


import click
from stochasticx.client.models import models
from stochasticx.client.datasets import datasets
from stochasticx.client.jobs import jobs
from stochasticx.client.auth import login, me
from stochasticx.client.deployments import instances, deployments
from stochasticx.client.inferences import inference


@click.group()
def cli():
    pass
    
 
cli.add_command(models)
cli.add_command(datasets)
cli.add_command(jobs)
cli.add_command(login)
cli.add_command(me)
cli.add_command(instances)
cli.add_command(deployments)
cli.add_command(inference)


import click
from stochasticx.models.models import Models
from stochasticx.datasets.datasets import Datasets
from stochasticx.jobs.jobs import (
    Job, 
    Jobs, 
    OptimizationCriteria, 
    SequenceClassificationTask, 
    QuestionAnsweringTask, 
    SummarizationTask, 
    TranslationTask, 
    TokenClassificationTask
)


@click.group(name="jobs")
def jobs():
    pass

@click.command(name="ls")
def ls_jobs():
    jobs = Jobs.get_jobs()
    for job in jobs:
        click.echo(job)


@click.command(name="inspect")
@click.option('--job_id', help='Job ID')
def job_inspect(job_id):
    job = Jobs.get_job(job_id)
    click.echo(job)


@click.command(name="status")
@click.option('--job_id', help='Job ID')
def job_status(job_id):
    job = Jobs.get_job(job_id)
    click.echo(job.get_status())
    


@click.group(name="launch")
def job_launch():
    pass


@click.command(name="sequence_classification")
@click.option('--job_name', help='Job name')
@click.option('--model_id', help='Model ID')
@click.option('--dataset_id', help='Dataset ID')
@click.option('--optimization_criteria', help='Optimization criteria. It should be latency or lossless')
@click.option('--sentence1_column', help='Sentence 1 column')
@click.option('--sentence2_column', help='Sentence 2 column')
@click.option('--labels_column', help='Labels column')
@click.option('--max_seq_length', default=128, help='Maximum sequence length')
def job_launch_sequence_classification(
    job_name,
    model_id, 
    dataset_id, 
    optimization_criteria,
    sentence1_column,
    sentence2_column,
    labels_column,
    max_seq_length
):
    model = Models.get_model(model_id)
    assert model is not None, "Model ID does not exist"
    dataset = Datasets.get_dataset(dataset_id)
    assert dataset is not None, "Dataset ID does not exist"
    assert optimization_criteria in ["latency", "lossless"], "optimization_criteria should take one of these values: latency or lossless"

    task_type = SequenceClassificationTask(
        sentence1_column=sentence1_column,
        sentence2_column=sentence2_column,
        labels_column=labels_column,
        max_seq_length=max_seq_length
    )

    job = Job(name=job_name)
    job.launch_auto(
        model=model,
        dataset=dataset,
        task_type=task_type,
        optimization_criteria=OptimizationCriteria.LATENCY
    )


@click.command(name="question_answering")
@click.option('--job_name', help='Job name')
@click.option('--model_id', help='Model ID')
@click.option('--dataset_id', help='Dataset ID')
@click.option('--optimization_criteria', help='Optimization criteria. It should be latency or lossless')
@click.option('--question_column', help='Question column')
@click.option('--answer_column', help='Answer column')
@click.option('--context_column', help='Context column')
@click.option('--max_seq_length', default=128, help='Maximum sequence length')
@click.option('--stride', default=30, help='Stride')
def job_launch_question_answering(
    job_name,
    model_id, 
    dataset_id, 
    optimization_criteria,
    question_column,
    answer_column,
    context_column,
    max_seq_length,
    stride
):
    model = Models.get_model(model_id)
    assert model is not None, "Model ID does not exist"
    dataset = Datasets.get_dataset(dataset_id)
    assert dataset is not None, "Dataset ID does not exist"
    assert optimization_criteria in ["latency", "lossless"], "optimization_criteria should take one of these values: latency or lossless"

    task_type = QuestionAnsweringTask(
        question_column=question_column,
        answer_column=answer_column,
        context_column=context_column,
        max_seq_length=max_seq_length,
        stride=stride
    )

    job = Job(name=job_name)
    job.launch_auto(
        model=model,
        dataset=dataset,
        task_type=task_type,
        optimization_criteria=OptimizationCriteria.LATENCY
    )


@click.command(name="summarization")
@click.option('--job_name', help='Job name')
@click.option('--model_id', help='Model ID')
@click.option('--dataset_id', help='Dataset ID')
@click.option('--optimization_criteria', help='Optimization criteria. It should be latency or lossless')
@click.option('--text_column', help='Text column')
@click.option('--summary_column', help='Summary column')
@click.option('--max_source_length', default=256, help='Maximum source length')
@click.option('--max_target_length', default=64, help='Maximum target length')
@click.option('--lang', default="en", help='Language')
@click.option('--pad_to_max_length', default=False, help='Pad to maximum length')
@click.option('--num_beams', default=4, help='Number of beams')
@click.option('--ignore_pad_token_for_loss', default=True, help='Ignore pad token for loss')
@click.option('--source_prefix', default="", help='Source prefix')
@click.option('--forced_bos_token', default=None, help='Forced BOS token')
def job_launch_summarization(
    job_name,
    model_id, 
    dataset_id, 
    optimization_criteria,
    text_column,
    summary_column,
    max_source_length,
    max_target_length,
    lang,
    pad_to_max_length,
    num_beams,
    ignore_pad_token_for_loss,
    source_prefix,
    forced_bos_token
):
    model = Models.get_model(model_id)
    assert model is not None, "Model ID does not exist"
    dataset = Datasets.get_dataset(dataset_id)
    assert dataset is not None, "Dataset ID does not exist"
    assert optimization_criteria in ["latency", "lossless"], "optimization_criteria should take one of these values: latency or lossless"

    task_type = SummarizationTask(
        text_column=text_column,
        summary_column=summary_column,
        max_source_length=max_source_length,
        max_target_length=max_target_length,
        lang=lang,
        pad_to_max_length=pad_to_max_length,
        num_beams=num_beams,
        ignore_pad_token_for_loss=ignore_pad_token_for_loss,
        source_prefix=source_prefix,
        forced_bos_token=forced_bos_token
    )

    job = Job(name=job_name)
    job.launch_auto(
        model=model,
        dataset=dataset,
        task_type=task_type,
        optimization_criteria=OptimizationCriteria.LATENCY
    )


@click.command(name="translation")
@click.option('--job_name', help='Job name')
@click.option('--model_id', help='Model ID')
@click.option('--dataset_id', help='Dataset ID')
@click.option('--optimization_criteria', help='Optimization criteria. It should be latency or lossless')
@click.option('--translation_column', help='Translation column')
@click.option('--src_lang', default="en", help='Source language')
@click.option('--tgt_lang', default="es", help='Target language')
@click.option('--max_source_length', default=256, help='Maximum source length')
@click.option('--max_target_length', default=256, help='Maximum target length length')
@click.option('--pad_to_max_length', default=False, help='Pad to maximum length')
@click.option('--num_beams', default=4, help='Number of beams')
@click.option('--ignore_pad_token_for_loss', default=True, help='Ignore pad token for loss')
@click.option('--source_prefix', default="", help='Source prefix')
@click.option('--forced_bos_token', default=None, help='Forced bos token')
def job_launch_translation(
    job_name,
    model_id, 
    dataset_id, 
    optimization_criteria,
    translation_column,
    src_lang,
    tgt_lang,
    max_source_length,
    max_target_length,
    pad_to_max_length,
    num_beams,
    ignore_pad_token_for_loss,
    source_prefix,
    forced_bos_token
):
    model = Models.get_model(model_id)
    assert model is not None, "Model ID does not exist"
    dataset = Datasets.get_dataset(dataset_id)
    assert dataset is not None, "Dataset ID does not exist"
    assert optimization_criteria in ["latency", "lossless"], "optimization_criteria should take one of these values: latency or lossless"

    task_type = TranslationTask(
        translation_column=translation_column,
        src_lang=src_lang,
        tgt_lang=tgt_lang,
        max_source_length=max_source_length,
        max_target_length=max_target_length,
        pad_to_max_length=pad_to_max_length,
        num_beams=num_beams,
        ignore_pad_token_for_loss=ignore_pad_token_for_loss,
        source_prefix=source_prefix,
        forced_bos_token=forced_bos_token
    )

    job = Job(name=job_name)
    job.launch_auto(
        model=model,
        dataset=dataset,
        task_type=task_type,
        optimization_criteria=OptimizationCriteria.LATENCY
    )


@click.command(name="token_classification")
@click.option('--job_name', help='Job name')
@click.option('--model_id', help='Model ID')
@click.option('--dataset_id', help='Dataset ID')
@click.option('--optimization_criteria', help='Optimization criteria. It should be latency or lossless')
@click.option('--tokens_column', help='Tokens column')
@click.option('--domains_column', help='Domains 2 column')
@click.option('--ner_tags_column', help='Ner column')
@click.option('--max_seq_length', default=128, help='Maximum sequence length')
@click.option('--label_all_tokens', default=False, help='Label all columns')
def job_launch_token_classification(
    job_name,
    model_id, 
    dataset_id, 
    optimization_criteria,
    tokens_column,
    domains_column,
    ner_tags_column,
    max_seq_length,
    label_all_tokens
):
    model = Models.get_model(model_id)
    assert model is not None, "Model ID does not exist"
    dataset = Datasets.get_dataset(dataset_id)
    assert dataset is not None, "Dataset ID does not exist"
    assert optimization_criteria in ["latency", "lossless"], "optimization_criteria should take one of these values: latency or lossless"

    task_type = TokenClassificationTask(
        tokens_column=tokens_column,
        domains_column=domains_column,
        ner_tags_column=ner_tags_column,
        max_seq_length=max_seq_length,
        label_all_tokens=label_all_tokens
    )

    job = Job(name=job_name)
    job.launch_auto(
        model=model,
        dataset=dataset,
        task_type=task_type,
        optimization_criteria=OptimizationCriteria.LATENCY
    )
    
    
job_launch.add_command(job_launch_sequence_classification)
job_launch.add_command(job_launch_question_answering)
job_launch.add_command(job_launch_summarization)
job_launch.add_command(job_launch_translation) 
job_launch.add_command(job_launch_token_classification)

jobs.add_command(ls_jobs)
jobs.add_command(job_inspect)
jobs.add_command(job_status)
jobs.add_command(job_launch)



from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.constants.urls import JOBS_URL
from stochasticx.models.models import Model, Models
from stochasticx.datasets.datasets import Dataset

from typing import List, Dict

import requests


class OptimizationCriteria:
    LATENCY = "latency"
    LOSSLESS = "lossless"

 
class TaskType:
    def __init__(self):
        pass
    
    def get_dataset_columns(self):
        raise NotImplementedError("This method should be implemented by the child class")
    
    def get_task_arguments(self):
        raise NotImplementedError("This method should be implemented by the child class")
    
    def get_task_type(self):
        raise NotImplementedError("This method should be implemented by the child class")

 
class SequenceClassificationTask(TaskType):
    def __init__(
        self,
        sentence1_column,
        sentence2_column,
        labels_column,
        max_seq_length=128
    ):
        self.sentence1_column = sentence1_column
        self.sentence2_column = sentence2_column
        self.labels_column = labels_column
        self.max_seq_length = max_seq_length
        
    def get_dataset_columns(self):
        return {
            "sentence1_column": self.sentence1_column,
            "sentence2_column": self.sentence2_column,
            "labels_column": self.labels_column
        }
    
    def get_task_arguments(self):
        return {
            "max_seq_length": self.max_seq_length
        }
        
    def get_task_type(self):
        return "sequence_classification"

 
class QuestionAnsweringTask(TaskType):
    def __init__(
        self,
        question_column,
        answer_column,
        context_column,
        max_seq_length=128,
        stride=30
    ):
        self.question_column = question_column
        self.answer_column = answer_column
        self.context_column = context_column
        self.max_seq_length = max_seq_length
        self.stride = stride
        
    def get_dataset_columns(self):
        return {
            "question_column": self.question_column,
            "answer_column": self.answer_column,
            "context_column": self.context_column
        }
    
    def get_task_arguments(self):
        return {
            "max_seq_length": self.max_seq_length,
            "stride": self.stride
        }
        
    def get_task_type(self):
        return "question_answering"

 
class SummarizationTask(TaskType):
    def __init__(
        self,
        text_column,
        summary_column,
        max_source_length=256,
        max_target_length=64,
	    lang="en",
        pad_to_max_length=False,
        num_beams=4,
        ignore_pad_token_for_loss=True,
        source_prefix="",
        forced_bos_token=None
    ):
        self.text_column = text_column
        self.summary_column = summary_column
        self.max_source_length = max_source_length
        self.max_target_length = max_target_length
        self.lang = lang
        self.pad_to_max_length = pad_to_max_length
        self.num_beams = num_beams
        self.ignore_pad_token_for_loss = ignore_pad_token_for_loss
        self.source_prefix = source_prefix
        self.forced_bos_token = forced_bos_token
        
    def get_dataset_columns(self):
        return {
            "text_column": self.text_column,
            "summary_column": self.summary_column
        }
    
    def get_task_arguments(self):
        return {
            "max_source_length": self.max_source_length,
            "max_target_length": self.max_target_length,
            "lang": self.lang,
            "pad_to_max_length": self.pad_to_max_length,
            "num_beams": self.num_beams,
            "ignore_pad_token_for_loss": self.ignore_pad_token_for_loss,
            "source_prefix": self.source_prefix,
            "forced_bos_token": self.forced_bos_token
        }
        
    def get_task_type(self):
        return "summarization"

  
class TranslationTask(TaskType):
    def __init__(
        self,
        translation_column,
        src_lang="en",
        tgt_lang="es",
        max_source_length=256,
        max_target_length= 256,
        pad_to_max_length=False,
        num_beams=4,
        ignore_pad_token_for_loss=True,
        source_prefix="",
        forced_bos_token=None
    ):
        self.translation_column = translation_column
        self.src_lang = src_lang
        self.tgt_lang = tgt_lang
        self.max_source_length = max_source_length
        self.max_target_length = max_target_length
        self.pad_to_max_length = pad_to_max_length
        self.num_beams = num_beams
        self.ignore_pad_token_for_loss = ignore_pad_token_for_loss
        self.source_prefix = source_prefix
        self.forced_bos_token = forced_bos_token
        
    def get_dataset_columns(self):
        return {
            "translation_column": self.translation_column
        }
    
    def get_task_arguments(self):
        return {
            "src_lang": self.src_lang,
            "tgt_lang": self.tgt_lang,
            "max_source_length": self.max_source_length,
            "max_target_length": self.max_target_length,
            "pad_to_max_length": self.pad_to_max_length,
            "num_beams": self.num_beams,
            "ignore_pad_token_for_loss": self.ignore_pad_token_for_loss,
            "source_prefix": self.source_prefix,
            "forced_bos_token": self.forced_bos_token
        }

    def get_task_type(self):
        return "translation"


class TokenClassificationTask(TaskType):
    def __init__(
        self,
        tokens_column,
        domains_column,
        ner_tags_column,
        max_seq_length=128,
        label_all_tokens=False
    ):
        self.tokens_column = tokens_column
        self.domains_column = domains_column
        self.ner_tags_column = ner_tags_column
        self.max_seq_length = max_seq_length
        self.label_all_tokens = label_all_tokens
        
    def get_dataset_columns(self):
        return {
            "tokens_column": self.tokens_column,
            "domains_column": self.domains_column,
            "ner_tags_column": self.ner_tags_column
        }
    
    def get_task_arguments(self):
        return {
            "max_seq_length": self.max_seq_length,
            "label_all_tokens": self.label_all_tokens
        }
        
    def get_task_type(self):
        return "token_classification"


class Job:
    def __init__(
        self,
        name
    ):
        self.id = None
        self.name = name
        self.original_model = None
        self.optimization_type = None
        self.optimization_criteria = None
        self.optimized_models = []
        self.job_status = None
        self.created_at = None
    
    def _populate_job(self, info_from_api):
        self.id = info_from_api.get("id")
        self.name = info_from_api.get("name")
        
        original_model_id = None 
        original_model_data = info_from_api.get("model")
        if original_model_data is not None and isinstance(original_model_data, dict):
            original_model_id = original_model_data.get("id")
        self.original_model = Models.get_model(original_model_id)
        
        self.optimization_type = info_from_api.get("optimizationType")
        self.optimization_criteria = info_from_api.get("optimizationCriteria")
        self.job_status = info_from_api.get("jobStatus")
        self.created_at = info_from_api.get("createdAt")
        
        if info_from_api.get("models") is not None and isinstance(info_from_api.get("models"), list):
            for optimized_model_id in info_from_api.get("models"):
                optimized_model = Models.get_optimized_model(optimized_model_id)
                self.optimized_models.append(optimized_model)
                
    def sync(self):
        if self.id is not None:
            temp_job = Jobs.get_job(self.id)
            self.name = temp_job.name
            self.original_model = temp_job.original_model
            self.optimization_type = temp_job.optimization_type
            self.optimization_criteria = temp_job.optimization_criteria
            self.optimized_models = temp_job.optimized_models
            self.job_status = temp_job.job_status
            self.created_at = temp_job.created_at
    
    def launch_auto(
        self,
        model: Model,
        dataset: Dataset,
        task_type: TaskType,
        optimization_criteria: str = OptimizationCriteria.LATENCY,
    ):
        assert model.is_uploaded, "This model has not been uploaded. Upload it first"
        assert dataset.is_uploaded, "This dataset has not been uploaded. Upload it first"
        assert isinstance(task_type, TaskType), "The task type is not valid"
        
        job_parameters = {
            "dataset": dataset.get_id(),
            "model": model.get_id(),
            "modelCloud": "s3",
            "jobName": self.name,
            "optimizationCriteria": optimization_criteria,
            "selectedColumns": task_type.get_dataset_columns(),
            "taskArguments": task_type.get_task_arguments(),
            "taskType": task_type.get_task_type(),
            "optimizationType": "auto",
            "finetuningArguments": {},
            "knowledgeDistillationArguments": {},
            "instanceType": "g4dn.xlarge",
            "isCPU": False,
            "isGPU": True
        }
        
        auth_header = AuthUtils.get_auth_headers()
        r = requests.post(
            JOBS_URL, 
            headers=auth_header, 
            json=job_parameters
        )
        r.raise_for_status()
        
        data = r.json().get("data")
        if data is not None:
            self.id = data.get("id")
        
    def get_status(self):
        self.sync()
        
        return self.job_status
        
    def __str__(self):
        return "Job ID: {} ; Name: {} ; Status: {} ; Created at: {} ; Optimization type: {} ; Optimization criteria: {}".format(
            self.id,
            self.name,
            self.job_status,
            self.created_at,
            self.optimization_type,
            self.optimization_criteria
        )
        

class Jobs:
    
    @staticmethod
    def get_jobs():
        jobs = []

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(JOBS_URL, headers=auth_header)
        jobs_data = response.json().get("data")
        
        if jobs_data is not None:
            for job_data in jobs_data:
                job = Job(
                    name=None
                )
                
                job._populate_job(job_data)
                jobs.append(job)
                
        return jobs
    
    @staticmethod
    def get_job(job_id):
        url = JOBS_URL + "/{}".format(job_id)

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(url, headers=auth_header)
        job_data = response.json().get("data")
        
        if job_data is not None:
            job_data = job_data.get("job")

        if job_data is not None:
            job = Job(
                name=job_data.get("name")
            )
            
            job._populate_job(job_data)
            
            return job
        
        return None


import logging
import datetime
import os
from pathlib import Path
import tempfile


class CustomFormatter(logging.Formatter):
    """Logging colored formatter"""

    grey = "\x1b[38;20m"
    green = '\x1b[92m'
    blue = '\x1b[38;5;39m'
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"

    def __init__(self, format):
        """Initializes the class

        Args:
            format: format of logger
        """
        super().__init__()
        self.fmt = format
        self.FORMATS = {
            logging.DEBUG: self.grey + self.fmt + self.reset,
            logging.INFO: self.green + self.fmt + self.reset,
            logging.WARNING: self.yellow + self.fmt + self.reset,
            logging.ERROR: self.red + self.fmt + self.reset,
            logging.CRITICAL: self.bold_red + self.fmt + self.reset
        }

    def format(self, record):
        """Given a record, format it

        Args:
            record: the record

        Returns:
            the formatted record
        """
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def configure_logger(name: str, fmt=None):
    """Configure local logger

    Args:
        name (str): python name file
        fmt (_type_, optional): -. Defaults to None.

    Returns:
        logger
    """
    # Create custom logger logging all five levels
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.propagate = False

    # Define format for logs
    if fmt is None:
        fmt = '%(asctime)s | %(levelname)s | %(name)s | %(message)s'

    # Create stdout handler for logging to the console (logs all five levels)
    stdout_handler = logging.StreamHandler()
    stdout_handler.setLevel(logging.DEBUG)
    stdout_handler.setFormatter(CustomFormatter(fmt))

    # Create file handler for logging to a file (logs all five levels)
    today = datetime.date.today()
    
    logs_dir = os.getenv("LOGS_SAVING_PATH")
    
    if logs_dir is None:
        logs_dir = tempfile.gettempdir()
    
    log_file_path = str(Path(logs_dir).resolve() / 'acceleron_{}.log'.format(today.strftime('%Y_%m_%d')))
    file_handler = logging.FileHandler(log_file_path)
    file_handler.setLevel(logging.DEBUG)
    file_handler.setFormatter(logging.Formatter(fmt))

    # Add both handlers to the logger
    logger.addHandler(stdout_handler)
    logger.addHandler(file_handler)

    return logger